package com.example.nto.controller;


import com.example.nto.dto.EmployeeDTO;
import com.example.nto.dto.EnterDTO;
import com.example.nto.entity.*;
import com.example.nto.service.impl.EmployeeServiceImpl;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.Authentication;
import org.springframework.web.bind.annotation.*;

import java.net.http.HttpClient;
import java.util.List;

@RestController
@RequiredArgsConstructor
@RequestMapping("/api")
public class EmployeeController {

    private final EmployeeServiceImpl employeeService;

    @GetMapping("/user/login")
    public ResponseEntity<EmployeeDTO> login(Authentication authentication){
        return ResponseEntity.ok(employeeService.getEmployeeByUsername(authentication.getName()));
    }

    @GetMapping("/user/info/{login}")
    public ResponseEntity<EmployeeDTO> info(@PathVariable("login") String username){
        return ResponseEntity.ok(employeeService.getInfo(username));
    }

    @PostMapping("/user/add/{login}")
    public ResponseEntity<EnterDTO> add(@PathVariable("login") String login, @RequestBody EnterDTO enterDTO){
        return ResponseEntity.ok(employeeService.addEnter(login, enterDTO));
    }

    @GetMapping("/user/list/{login}")
    public List<EnterDTO> list(@PathVariable("login") String username){
        return employeeService.getAllEmployeeEnters(username);
    }

    @PatchMapping("/admin/authority/change/{login}")
    public ResponseEntity<HttpStatus> authChange(@PathVariable("login") String username, @RequestBody Authority authority){
        return ResponseEntity.ok(employeeService.changeAuthority(username, authority));
    }
}
