package com.example.nto.service.impl;

import com.example.nto.dto.EmployeeDTO;
import com.example.nto.dto.EnterDTO;
import com.example.nto.entity.*;
import com.example.nto.exception.EmployeeNotFoundException;
import com.example.nto.exception.IncorrectAuthority;
import com.example.nto.exception.IncorrectCodeException;
import com.example.nto.exception.IncorrectTypeException;
import com.example.nto.repository.*;
import com.example.nto.service.EmployeeService;
import com.example.nto.util.EmployeeMapper;
import com.example.nto.util.EnterMapper;
import lombok.RequiredArgsConstructor;
import org.springframework.http.HttpStatus;
import org.springframework.stereotype.Service;

import java.util.List;
import java.util.Optional;
import java.util.stream.Collectors;


@Service
@RequiredArgsConstructor
public class EmployeeServiceImpl implements EmployeeService {

    private final EmployeeRepository employeeRepository;
    private final CodeRepository codeRepository;
    private final EnterTypeRepository enterTypeRepository;
    private final EnterRepository enterRepository;
    private final AuthorityRepository authorityRepository;
    private final EmployeeAuthorityRepository employeeAuthorityRepository;

    @Override
    public EmployeeDTO getEmployeeByUsername(String username) {
        Optional<Employee> optionalEmployee = employeeRepository.findByUsername(username);

        if(optionalEmployee.isEmpty())
            throw new EmployeeNotFoundException("Employee with username: " + username + " not found");

        return EmployeeMapper.convertToDTO(optionalEmployee.get());
    }

    @Override
    public EmployeeDTO getInfo(String username) {
        Optional<Employee> optionalEmployee = employeeRepository.findByUsername(username);
        if(optionalEmployee.isEmpty()) {
            throw new EmployeeNotFoundException("Employee with username: " + username + " not found");
        }

        return EmployeeMapper.convertToDTO(optionalEmployee.get());
    }

    @Override
    public EnterDTO addEnter(String username, EnterDTO enterDTO) {
        Optional<Employee> optionalEmployee = employeeRepository.findByUsername(username);
        if(optionalEmployee.isEmpty())
            throw new EmployeeNotFoundException("Employee with username: " + username + " not found");
        Optional<Code> optionalCode = codeRepository.findByValue(enterDTO.getValue());
        if(optionalCode.isEmpty())
            throw new IncorrectCodeException("Incorrect code");

        Optional<EnterType> optionalEnterType = enterTypeRepository.findByType(enterDTO.getType());
        if(optionalEnterType.isEmpty())
            throw new IncorrectTypeException("Incorrect type");

        Enter enter = new Enter();
        enter.setType(optionalEnterType.get());
        enter.setTime(enterDTO.getTime());
        enter.setValue(enterDTO.getValue());
        enter.setUsername(username);


        return EnterMapper.convertToDTO(enterRepository.save(enter));
    }

    @Override
    public List<EnterDTO> getAllEmployeeEnters(String username) {
        return enterRepository.findAllByUsername(username).stream()
                .map(EnterMapper::convertToDTO)
                .collect(Collectors.toList());
    }

    @Override
    public HttpStatus changeAuthority(String username, Authority auth) {
        Optional<Employee> optionalEmployee = employeeRepository.findByUsername(username);
        if(optionalEmployee.isEmpty())
            throw new EmployeeNotFoundException("Employee with username: " + username + " not found");
        Optional<Authority> optionalAuthority = authorityRepository.findByAuthority(auth.getAuthority());
        if(optionalAuthority.isEmpty())
            throw new IncorrectAuthority("Incorrect authority");

        EmployeeAuthority employeeAuthority = employeeAuthorityRepository.findByEmpId(optionalEmployee.get());
        employeeAuthority.setAuthId(optionalAuthority.get());
        employeeAuthorityRepository.save(employeeAuthority);
        return HttpStatus.OK;
    }


}
